module.exports = (node, graph) => {
  var ctx = graph.ctx
  
  var createPlane = require('primitive-plane')
  var computeEdges = require('geom-edges')
  
  var sx = node.in('sx', 1)
  var sy = node.in('sy', 1)
  var nx = node.in('nx', 1, { precision: 0 })
  var ny = node.in('ny', 1, { precision: 0 })
  var wireframe = node.in('wireframe', false)
  var quads = node.in('quads', false)
  var axis = node.in('axis', 'XY', { type: 'dropdown', values: ['XY', 'XZ'] })
  
  var geometry = node.out('geometry')
  
  function update () {
    var g = createPlane(sx.value, sy.value, nx.value, ny.value, {
      quads: quads.value
    })
    if (wireframe.value) {
      g.cells = computeEdges(g.cells)
    }
    if (axis.value === 'XZ') {
      g.positions = g.positions.map((p) => ([p[0], p[2], -p[1]]))
      g.normals = g.normals.map(() => [0, 1, 0])
    }
    geometry.setValue(g)
  }
  
  update()
  
  sx.onChange = update
  sy.onChange = update
  nx.onChange = update
  ny.onChange = update
  wireframe.onChange = update
  quads.onChange = update
  axis.onChange = update
};